#ifndef __EZFPGA_H__
#define __EZFPGA_H__

/*
  Building configration
*/

#ifndef EZFPGA_STATIC_LIB
  #define EZFPGA_STATIC_LIB   1 /* If 0 then use ezfpga.DLL   */
#endif
#ifndef EZFPGA_CONSOLE_FUNC
  #define EZFPGA_CONSOLE_FUNC 1 /* show Remain on console     */
#endif
#ifndef EZFPGA_JMEM_FUNC
  #define EZFPGA_JMEM_FUNC    1 /* JTAG Memory Access Function */
#endif

/* 'port_type' macros */
#define DEVICE_USB2(no)      ((0x00010000)+(no))
#define DEVICE_XPAR(PortAdd) ((0x00020000)+(PortAdd))

/* config remain & xfer size flag */
#define EZFPGA_NON_BLOCK   0x80000000 /* non-blocking mode    */

/* type of error code */
typedef DWORD ECODE;

/* error code bit assign */
/* System error */
#define ERR_DRIVER   0x80000000 /* DLL Driver error     */
#define ERR_PROBE    0x40000000 /* Probe                */
#define ERR_COMMUNI  0x20000000 /* Communication error  */
#define ERR_FILE     0x10000000 /* file error           */
#define ERR_BUSY     0x08000000 /* driver BUSY          */
/* Target error */
#define ERR_TARGET   0x00800000 /* no Target found      */
#define ERR_POWER    0x00400000 /* targer power down    */
/* Data */
#define ERR_FORMAT   0x00008000 /* data/parameter format error */
/* Configration error */
#define ERR_DEVICE   0x00000008 /* Device not match     */
#define ERR_SHUTDOWN 0x00000004 /* can't shurdown       */
#define ERR_DONE     0x00000002 /* DONE signal inavtive */
#define ERR_CRC      0x00000001 /* CRC error            */

#ifndef EZFPGA_LIB_BUILD
/* ------------------- EZFPGA Core Functions ------------------- */

/* Initialize */
extern DWORD ezfpga_init(DWORD port_type);

/* unlink DLL (DLL only) */
void ezfpga_unlink(void);

#if EZFPGA_STATIC_LIB
/* Configration */
extern DWORD ezfpga_config(const char *cfg_data,DWORD *remain);

/* Communication */
extern DWORD ezfpga_spi_xfer(BYTE port,BYTE *txbuf,BYTE *rxbuf,DWORD *size);

/* option : read status */
extern DWORD ezfpga_get_status(DWORD *deviceid,DWORD *status,DWORD *usercode);

/* option : free probe resource */
extern DWORD ezfpga_free(void);

/* option : shutdown FPGA */
extern DWORD ezfpga_shutdown(void);

#else /* EZFPGA_STATIC_LIB */
 /* -------------------- DLL link -------------------- */

/* DLL Export prefix */
#define DLLEXPORT __declspec(dllexport)
#ifdef __MINGW32__
#define DLLAPI
#else
#define DLLAPI WINAPI
#endif

DWORD (DLLAPI *ezfpga_config)(const char *cfg_data,DWORD *remain);
DWORD (DLLAPI *ezfpga_spi_xfer)(BYTE port,BYTE *txbuf,BYTE *rxbuf,DWORD *num_bytes);
DWORD (DLLAPI *ezfpga_get_status)(DWORD *deviceid,DWORD *status,DWORD *usercode);
DWORD (DLLAPI *ezfpga_free)(void);
DWORD (DLLAPI *ezfpga_shutdown)(void);

#endif /* EZFPGA_STATIC_LIB */

/* ------------------- EZFPGA Console show option ------------------- */
#if EZFPGA_CONSOLE_FUNC
/* console show */
DWORD ezfpga_config_show(const char *config_data,DWORD size);
DWORD ezfpga_xfer_show(BYTE port,BYTE *txbuf,BYTE *rxbuf,DWORD size);
#endif

/* ------------------- EZFPGA memory Access option ------------------- */
#if EZFPGA_JMEM_FUNC

DWORD jmem_write(DWORD address,const BYTE *data,DWORD size);
DWORD jmem_read(DWORD address,BYTE *data,DWORD size);
DWORD jmem_verify(DWORD address,BYTE *data,DWORD size);

#endif

#endif /* EZFPGA_LIB_BUILD */

#endif /* __EZFPGA_H__ */
